#ifndef BOM_H_INCLUDED
#define BOM_H_INCLUDED

#include "ComTyp.h"



#if 0   // these are in ComTyp.h
 #define iMCS51   0           // ToolSet Definitions
 #define iMCS251  1
 #define iMCS166  2
 #define iMCSARM  3           // Gnu-ARM tools
#endif

/*
 * Common Sim/Target Stuff
 */

#ifdef __cplusplus
  extern "C"  {
#endif



//--- Types
#define  bt_void      0
#define  bt_bit       1
#define  bt_char      2
#define  bt_uchar     3
#define  bt_int       4
#define  bt_uint      5
#define  bt_short     6
#define  bt_ushort    7
#define  bt_long      8
#define  bt_ulong     9
#define  bt_float    10
#define  bt_double   11
#define  bt_ptr      12
#define  bt_union    13
#define  bt_struct   14
#define  bt_func     15
#define  bt_string   16
#define  bt_enum     17
#define  bt_field    18
#define  bt_int64    19
#define  bt_uint64   20



/*
 * Storage class specifiers
 */

#define  sc_strsym    0
#define  sc_static    1
#define  sc_auto      2
#define  sc_regi      3
#define  sc_global    4
#define  sc_extern    5
#define  sc_type      6
#define  sc_const     7
#define  sc_member    8
#define  sc_label     9
#define  sc_ulabel   10
#define  sc_parm     11
#define  sc_sfr      12
#define  sc_abit     13
#define  sc_tag      14
#define  sc_bind     15


/*
 * Expression - Node types
 */

#define  x_const     1    /* some constant */
#define  e_strcon    2    /* string object */
#define  e_labcon    3    /* static object */
#define  e_autcon    4    /* local object  */
#define  e_namcon    5    /* dScope function reference */
#define  e_simvar    6    /* dScope predefined name */
#define  e_regid     7    /* some Register name */
#define  e_vtreg     8    /* some vtreg */
#define  e_usrcon    9    /* namcon of user program */
#define  e_vcon      10   /* namcon of a vSym */

#define  e_ref       11   /* some reference */
#define  e_reff      12   /* bit-field-reference */
#define  e_cast      13   /* some cast */
#define  e_postinc   14   /* ++ (postfix !) */
#define  e_postdec   15   /* -- (postfix !) */
#define  e_addr      16   /* & (addr_of) */
#define  e_cont      17   /* * (content_of) */
#define  e_plus      18   /* unary + */
#define  e_minu      19   /* unary - */
#define  e_not       20   /* ! (not) */
#define  e_compl     21   /* ~ (complement) */
#define  e_asn       22   /* = (assign to) */
#define  e_asnor     23   /* |= */
#define  e_asnxor    24   /* ^= */
#define  e_asnand    25   /* &= */
#define  e_asnlsh    26   /* <<= */
#define  e_asnrsh    27   /* >>= */
#define  e_asnadd    28   /* += */
#define  e_asnsub    29   /* -= */
#define  e_asnmul    30   /* *= */
#define  e_asndiv    31   /* /= */
#define  e_asnmod    32   /* %= */
#define  e_hook      33   /* e0 ? e1 : e2 */
#define  e_land      34   /* && */
#define  e_lor       35   /* || */
#define  e_and       36   /* & (bit-and) */
#define  e_or        37   /* | (bit-or) */
#define  e_xor       38   /* ^ (bit-xor) */
#define  e_equ       39   /* == */
#define  e_nequ      40   /* != */
#define  e_lequ      41   /* <= */
#define  e_gequ      42   /* >= */
#define  e_gt        43   /* >  */
#define  e_lt        44   /* <  */
#define  e_lsh       45   /* << */
#define  e_rsh       46   /* >> */
#define  e_add       47   /* +  */
#define  e_sub       48   /* -  */
#define  e_mod       49   /* %  */
#define  e_div       50   /* /  */
#define  e_mul       51   /* *  */
#define  e_preinc    52   /* ++ (prefix !) */
#define  e_predec    53   /* -- (prefix !) */
#define  e_indx      54   /* [] */

#define  e_comma     55   /* expr, expr */
#define  e_param     56   /* actual parameter */
#define  e_void      57   /* void-node */
#define  e_fcall     58   /* direct function call */
#define  e_size      59   /* sizeof (expr) */
#define  e_move      60   /* some memcpy */
#define  e_struid    61   /* 'struct -> id' or 'struct.id' */
#define  e_lea       62   /* load effective address */
#define  e_exec      63   /* exec ("command") */



#pragma pack(1)

/*
 * mSpaces (F66, 166/167)
 */

#define  xNEAR        1
#define  xFAR         2
#define  xHUGE        3
#define  xXHUGE       4

#define  mDATA        1         // fox _near
#define  mIDATA       2         // fox _idata
#define  mPDATA       3         // fox _shuge
#define  mXDATA       4         // fox _sdata
#define  mBDATA       5         // fox _bdata
#define  mECODE       6         // fox code-space


/*
 * nfh:   0 := ndata-ptr (16-Bit, page2:pof14)
 *        1 := far-ptr   (32-Bit, seg16:sof16)
 *        2 := huge-ptr  (32-Bit, linear)
 *        3 := xhuge-ptr (32-Bit, linear)
 */

typedef struct typ {            /* Structure for a Type */
  BYTE              type;       /* bt_* */
  DWORD             size;       /* Size of Item described */

  DWORD32     v_flag : 1;       /* 1 on Array | Struct | Func */
  DWORD32        vpf : 1;       /* voidpointer */
  DWORD32       duse : 2;       /* 1=const, 2=volatile, 3=both */
  DWORD32        nfh : 4;       /* 1:=near, 2:=far, 3:=huge, 7=short */
  DWORD32        tps : 1;       // ptr-to-char for 'e_strcon'
  DWORD32      iduse : 2;
  DWORD32      parmb : 2;       /* 1=f(void), 2=f(), 3=f(p,...)' */
  DWORD32      tType : 1;       /* a Typedef-Type ! */
  DWORD32      zTemp : 1;
  DWORD32     SpcSpc : 1;       /* explicit spaced pointer */
  DWORD32     eSpace : 1;       /* explicit mSpace given to spaced-ptr */

  BYTE             space;       /* mspace on pointers */
  BYTE            tspace;       /* typedef'd Space ! */
  BYTE            params;       /* Function: Nr. of Parameters */

  struct typ        *btp;       /* type-Link */
  struct sym       *head;       /* subtable struct/union/func */
  char            *sname;       /* ptr to type-Name */
} TYP;


#pragma pack()


//--- Basic-types
extern TYP tp_void;    // = { bt_void,   { 0, 0, 0, 0 }, 0, 0, 0,  };
extern TYP tp_bit;     // = { bt_bit,    { 0, 0, 0, 0 }, 0, 0, 1,  };
extern TYP tp_char;    // = { bt_char,   { 0, 0, 0, 0 }, 0, 0, 1,  };
extern TYP tp_uchar;   // = { bt_uchar,  { 0, 0, 0, 0 }, 0, 0, 1,  };
extern TYP tp_int;     // = { bt_int,    { 0, 0, 0, 0 }, 0, 0, 2,  };
extern TYP tp_uint;    // = { bt_uint,   { 0, 0, 0, 0 }, 0, 0, 2,  };
extern TYP tp_short;   // = { bt_short,  { 0, 0, 0, 0 }, 0, 0, 2,  };
extern TYP tp_ushort;  // = { bt_ushort, { 0, 0, 0, 0 }, 0, 0, 2,  };
extern TYP tp_long;    // = { bt_long,   { 0, 0, 0, 0 }, 0, 0, 4,  };
extern TYP tp_ulong;   // = { bt_ulong,  { 0, 0, 0, 0 }, 0, 0, 4,  };
extern TYP tp_float;   // = { bt_float,  { 0, 0, 0, 0 }, 0, 0, 4,  };
extern TYP tp_double;  // = { bt_double, { 0, 0, 0, 0 }, 0, 0, 8,  };
extern TYP tp_func;    // = { bt_func,   { 1, 0, 0, 0 }, 0, 0, 0,  };
extern TYP tp_funcN;   // = { bt_func,   { 1, 0, 0, 0 }, 0, 0, 0,  };
extern TYP tp_funcF;   // = { bt_func,   { 1, 0, 0, 0 }, 0, 0, 0,  };
extern TYP tp_funcS;   // = { bt_func,   { 1, 0, 0, 7 }, 0, 0, 0,  };

extern TYP tp_culong;  // = { bt_ulong,    0, 0, 1, 0, 0, 0, 4,  }; // const !


/*
 * Fox-Compound-Types
 */

#define ZT_CLIST       0x20     /* OMF166: component list descriptor */
#define ZT_PTR         0x21     /* OMF166: pointer descriptor        */
#define ZT_ARRAY       0x22     /* OMF166: array descriptor          */
#define ZT_FUNC        0x23     /* OMF166: function descriptor       */
#define ZT_STRU        0x24     /* OMF166: struct/union descriptor   */
#define ZT_FIELD       0x25     /* OMF166: bitfield descriptor       */
#define ZT_TYPTAG      0x26     /* Fox-Typedef */
#define ZT_COMP        0x27     /* Fox-Component */
#define ZT_FCLIST      0x28     /* Fox-Component-List */
#define ZT_FSTRU       0x29     /* Fox-struct/union descriptor */


/*
 * Fox Scalar-Types
 */

#define TI_NIL         0x40     /* untyped */
#define TI_BIT         0x41     /* bit */
#define TI_CHAR        0x42     /* char */
#define TI_UCHAR       0x43     /* uchar */
#define TI_INT         0x44     /* int */
#define TI_UINT        0x45     /* uint */
#define TI_LONG        0x46     /* long */
#define TI_ULONG       0x47     /* ulong */
#define TI_FLOAT       0x48     /* float (32-bit)  */
#define TI_DOUBLE      0x49     /* double (64-bit) */
#define TI_VOID        0x4A     /* void */
#define TI_LABEL       0x4B     /* label (near) */
#define TI_BITWORD     0x4C     /* a166 bitword */
#define TI_NEAR        0x4D     /* a166 near */
#define TI_FAR         0x4E     /* a166 far */
#define TI_DATA3       0x4F     /* a166 data3 */
#define TI_DATA4       0x50     /* a166 data4 */
#define TI_DATA8       0x51     /* a166 data8 */
#define TI_DATA16      0x52     /* a166 data16 */
#define TI_INTNO       0x53     /* a166 intno */
#define TI_REGBANK     0x54     /* a166 regbank */
#define TI_IFAR        0x55     /* a166 interrupt far */
#define TI_DATA32      0x56     /* a166 data32 */
#define TI_DATA64      0x57     /* a166 data64 */
#define TI_FRACT16     0x58     /* fractional-16 type */
#define TI_FRACT32     0x59     /* fractional-32 type */
#define TI_SHORT       0x7F     /* dummy type for Jumps */

/*
 * Dll-Query structure
 */

typedef struct qdll  {
  HWND       parent;        // parent window handle
  char     *pathUv3;        // path of Target-DLL
  HMODULE     Linst;        // DLL Instance handle

  int       ValSize;        // size of 'value[]' Buffer
  char         *key;        // Register key for driver DLL's
  char       *value;        // some command line, for example
                            // more info could be here ...
  char     *prjname;        // project path+name
} QDLL;

#define SZ_R166   (OFO (struct Reg166, macc))
#define SZ_ST10   (sizeof (struct Reg166))
#define SZ_RX51   (sizeof (struct Regx51))
#define SZ_SLSZ   (sizeof (struct RegS6))
#define SZ_S251   (sizeof (struct Reg251))
#define SZ_51MX   (sizeof (struct Reg51MX))
#define SZ_RARM   (sizeof (struct RegARM))


/*
 * iMCS-51 SFR-Attribute codes
 */

#define nSF   0x0000      // SFR not present
#define iSF   0x8000      // SFR is present
#define pSF   0x4000      // SFR is present and is Port-0...3
#define cSF   0x2000      // ECO 'external SFR' (EEPROM,TIMER,UART)
#define dSF   0x1000      // 80320: Sfr is DPL/DPH or DPL1/DPH1
// Note: the above need to be combined with ATR_READ/ATR_WRITE
//       for selectively attribute configuration
//       iAV replaced by ATR_READ/ATR_WRITE


/*
 * Memory spaces
 */

#define   mmNONE   0x0000           // not spaced
#define   mmXDATA  0x0001           // XDATA
#define   mmBANK0  0x0080           // BANK0
#define   mmBANK31 0x009F           // BANK31
#define   mmDATA   0x00F0           // DATA
#define   mmBIT    0x00F1           // BIT
#define   mmEDATA  0x00F2           // EDATA (i251)
#define   mmIDATA  0x00F3           // IDATA
#define   mmECODE  0x00F4           // 251 ecode
#define   mmHDATA  0x00F5           // 251 hdata
#define   mmHCONS  0x00F6           // 251 hconst
#define   mmCONST  0x00F7           // 251 const
#define   mmPDATA  0x00FE           // PDATA (c51 macht das bei generic)
#define   mmCODE   0x00FF           // CODE
#define   mmPHYS   0x0100           // Physical SLE66+ Memory
#define   mmPHLIM  0x0110           // 1MB Physical Memory

#define   mmU1MEM  0x00F8           // UsrMem #1 (E2-PROM)
#define   mmU2MEM  0x00F9           // UsrMem #2 (undefined)
#define   mmU3MEM  0x00FA           // UsrMem #3 (undefined)
#define   mmU4MEM  0x00FB           // UsrMem #4 (undefined)
#define   mmU5MEM  0x00FC           // UsrMem #5 (undefined)
#define   mmU6MEM  0x00FD           // UsrMem #6 (undefined)

#define   mmSFR000 0x0200           // SFR-Bank 0...255
#define   mmSFR255 0x02FF           // SFR-Bank 255 (D:255)



//--------------------------------------------------------------

#define iSLE88     100     // dead !
#define i80x51     0x00    // generic 80x51
#define iEXTMAC    0x01    // MAC167
#define iSLE66     0x02    // SLE66
#define iC51MX     0x03    // Philips 80C51MX
#define iRF5016    0x04    // Philips P8RF5016 (Mifare pro X)
#define iSMARTMX   0x05    // Philips SmartMX


#define xxSFRMUX  UsrDwrd[13]  // holds 'SfrMux' ((BYTE *) &SfrMux[0])  /SLE66/

#define xxDCIPRM  UsrDwrd[14]  // holds 'DWORD &_dciUV2' for DCI Interface
#define Dio    ((struct dciUV2 *)(pio->UsrDwrd[14]))->dio
#define Dconn  ((struct dciUV2 *)(pio->UsrDwrd[14]))->dconn


#define   CMD_EXTCOVER   0x1000
// S8051/S166: Extended Coverage:
//      DWORD bR = ioc.SimFunc (CMD_EXTCOVER, char *"filename", DWORD nFlags);
//   bR:      1:=Ok, 0:=Error (file open/close/io)
//   nFlags:  1:=show jump-coverage details (like 'coverage details')



/*
 * DCI command messages (uses Windows MSG structure)
 */

#define DCI_BUILDPRJ        0x1000     // build current project
#define DCI_GETBUILDSTATUS  0x1001     // get current project build status
#define DCI_OPENPROJECT     0x1002     // open project
#define DCI_CLOSEPROJECT    0x1003     // close project


//--- DCI-Command return values via 'pAck':


#define DCI_CMDOK           (1) // command was Ok.
#define DCI_CMDNOTALLOWED   (2) // command cannot be executed now
#define DCI_BLDERRORS       (3) // previous build-project had error(s)
#define DCI_CMDFAILED       (4) // dci command failed (e.g. open project)

#define DCI_DONTKNOW       (-1) // don't know (example build-status)
#define DCI_UV2READY       (-2) // Uv2 booted (Uv2 - not debug mode)
#define DCI_UV2DEBUG       (-3) // Uv2 debug modus


/*
 * Flash-Capabilities provided by Target-DLL (Monxxx.dll)
 */
#define CAP_WRITE   0x0001    // Flash-Write (DownLoad)
#define CAP_ERASE   CAP_WRITE    // Flash-Erase



#ifdef __cplusplus
  }                           // End of 'extern "C"'
#endif

#endif
