#include "precomp.h"

CAgdiProgress::CAgdiProgress(long lAlreadyDone, long lActualSize, pCBF pCbFunc)
    : m_pTaskbarList(nullptr), m_lAlreadyDone(lAlreadyDone), m_lActualSize(lActualSize), m_pCbFunc(pCbFunc)
{
    if(FAILED(CoCreateInstance(CLSID_TaskbarList, nullptr, CLSCTX_SERVER, IID_ITaskbarList3, reinterpret_cast<void**>(&m_pTaskbarList))))
        m_pTaskbarList = nullptr;
    if(m_pTaskbarList)
        m_pTaskbarList->HrInit();
    if(m_lActualSize <= 0)
        m_lActualSize = 1;
}

CAgdiProgress::~CAgdiProgress()
{
    if(m_pTaskbarList)
    {
        m_pTaskbarList->SetProgressState(hMfrm, TBPF_NOPROGRESS);
        m_pTaskbarList->Release();
    }
}

IFACEMETHODIMP
CAgdiProgress::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_ISmartICE04Progress))
            *ppvObject = static_cast<ISmartICE04Progress*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CAgdiProgress::AddRef()
{
    return(2);
}

IFACEMETHODIMP_(ULONG)
CAgdiProgress::Release()
{
    return(1);
}

IFACEMETHODIMP
CAgdiProgress::Start()
{
    if(m_pTaskbarList)
    {
        m_pTaskbarList->SetProgressState(hMfrm, TBPF_NORMAL);
        m_pTaskbarList->SetProgressValue(hMfrm, 0ULL, 100ULL);
    }

    return(S_OK);
}

IFACEMETHODIMP
CAgdiProgress::Stop()
{
    if(m_pTaskbarList)
        m_pTaskbarList->SetProgressState(hMfrm, TBPF_NOPROGRESS);

    return(S_OK);
}

IFACEMETHODIMP
CAgdiProgress::SetProgress(long lCompleted, long lTotal)
{
    OIL ProgressBar;
    ProgressBar.Job   = PROGRESS_SETPOS;
    ProgressBar.pos   = std::min(100L, ((lCompleted + m_lAlreadyDone) * 100L) / std::max(m_lActualSize, lTotal));
    ProgressBar.low   = 0;
    ProgressBar.hig   = 100;
    ProgressBar.label = nullptr;
    m_pCbFunc(AG_CB_PROGRESS, &ProgressBar);

    if(m_pTaskbarList)
        m_pTaskbarList->SetProgressValue(hMfrm, static_cast<unsigned long>(lCompleted), static_cast<unsigned long>(lTotal));

    return(S_OK);
}

IFACEMETHODIMP
CAgdiProgress::get_Canceled(VARIANT_BOOL* pblCanceled)
{
    if(pblCanceled)
        *pblCanceled = VARIANT_FALSE;

    return(S_OK);
}
