#include "precomp.h"

HRESULT
CBinaryFile::Load(LPCWSTR filename, SAFEARRAY* psa)
{
    LARGE_INTEGER   size;
    HRESULT         hr;
    HANDLE          hFile;
    DWORD           dw;
    void*           pData;
    long            lOffset = 0;

    if(INVALID_HANDLE_VALUE == (hFile = CreateFileW(filename, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, nullptr, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, nullptr)))
        return(HRESULT_FROM_WIN32(GetLastError()));

    if(!GetFileSizeEx(hFile, &size))
    {
        hr = HRESULT_FROM_WIN32(GetLastError());
    }
    else if(size.HighPart)
    {
        hr = __HRESULT_FROM_WIN32(ERROR_FILE_TOO_LARGE);
    }
    else if(SUCCEEDED(hr = RedimAndLockArray(psa, lOffset, size.LowPart)))
    {
        if(SUCCEEDED(hr = SafeArrayPtrOfIndex(psa, &lOffset, &pData)))
        {
            if(!ReadFile(hFile, pData, size.LowPart, &dw, nullptr))
            {
                hr = HRESULT_FROM_WIN32(GetLastError());
            }
            else if(dw != size.LowPart)
            {
                hr = E_FAIL;
            }
        }
        SafeArrayUnlock(psa);
    }

    CloseHandle(hFile);
    return(hr);
}
