#include "precomp.h"

CDownloadSetup::CDownloadSetup()
    : m_dpiX(USER_DEFAULT_SCREEN_DPI), m_dpiY(USER_DEFAULT_SCREEN_DPI), m_dpiChanging(0)
{
    ZeroMemory(&m_ptMinTrackSize, sizeof(m_ptMinTrackSize));
}

HRESULT
CDownloadSetup::DoModal(HWND hWndParent)
{
    CThreadDpiAwarenessContext  dpiAwarness(DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2);
    CDownloadSetup              dlg;
    CActCtx                     ctx(IDR_MANIFEST);
    const INT_PTR               n = DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_DOWNLOADSETUP), hWndParent, DlgProc, reinterpret_cast<LPARAM>(&dlg));
    return((n == -1) ? HRESULT_FROM_WIN32(GetLastError()) : ((n == IDOK) ? S_OK : S_FALSE));
}

INT_PTR
CDownloadSetup::OnWmInitDialog(HWND hWnd, WPARAM, LPARAM)
{
    RECT rc;
    m_dpiX = m_dpiY = MyGetDpiForWindow(hWnd);
    m_dpiChanging = 0;
    InstallDpiChangeMonitor(hWnd, WM_APP);
    GetClientRect(hWnd, &rc);
    OnWmSize(hWnd, SIZE_RESTORED, MAKELPARAM(rc.right - rc.left, rc.bottom - rc.top));
    GetWindowRect(hWnd, &rc);
    m_ptMinTrackSize.x = rc.right - rc.left;
    m_ptMinTrackSize.y = rc.bottom - rc.top;
    CheckDlgButton(hWnd, IDC_FLASH_ERASE, static_cast<UINT>((MonConf.Opt & FLASH_ERASE) ? BST_CHECKED : BST_UNCHECKED));
    CheckDlgButton(hWnd, IDC_FLASH_PROGRAM, static_cast<UINT>((MonConf.Opt & FLASH_PROGRAM) ? BST_CHECKED : BST_UNCHECKED));
    CheckDlgButton(hWnd, IDC_FLASH_VERIFY, static_cast<UINT>((MonConf.Opt & FLASH_VERIFY) ? BST_CHECKED : BST_UNCHECKED));
    return(TRUE);
}

INT_PTR
CDownloadSetup::OnWmCommand(HWND hWnd, WPARAM wParam, LPARAM)
{
    switch(LOWORD(wParam))
    {
        case IDCANCEL:
            OnWmInitDialog(hWnd, 0, reinterpret_cast<LPARAM>(this));
        case IDOK:
            MonConf.Opt &= ~(FLASH_ERASE | FLASH_PROGRAM | FLASH_VERIFY);

            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_FLASH_ERASE))
                MonConf.Opt |= FLASH_ERASE;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_FLASH_PROGRAM))
                MonConf.Opt |= FLASH_PROGRAM;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_FLASH_VERIFY))
                MonConf.Opt |= FLASH_VERIFY;

            EndDialog(hWnd, LOWORD(wParam));
            break;
        default:
            return(FALSE);
    }

    return(TRUE);
}

INT_PTR
CDownloadSetup::OnWmGetMinMaxInfo(HWND, WPARAM, LPARAM lParam)
{
    if(!m_dpiChanging)
        reinterpret_cast<MINMAXINFO*>(lParam)->ptMinTrackSize = m_ptMinTrackSize;

    return(TRUE);
}

INT_PTR
CDownloadSetup::OnWmSize(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
    if(wParam != SIZE_MINIMIZED)
    {
        RECT rc;
        SIZE margin;
        
        GetWindowRect(GetDlgItem(hWnd, IDC_DOWNLOAD_SETUP_GROUP), &rc);
        RectScreenToClient(hWnd, &rc);
        margin.cx = rc.left;
        margin.cy = rc.top;

        HWND hCtl = GetDlgItem(hWnd, IDCANCEL);
        GetWindowRect(hCtl, &rc);

        rc.right -= rc.left;
        rc.bottom -= rc.top;
        rc.left = LOWORD(lParam) - (margin.cx + rc.right);
        rc.top = HIWORD(lParam) - (margin.cy + rc.bottom);

        CDeferWindowPos pos(3);
        pos.MoveWindow(hCtl, rc.left, rc.top, rc.right, rc.bottom, TRUE);
        pos.MoveWindow(GetDlgItem(hWnd, IDOK), rc.left - (margin.cx + rc.right), rc.top, rc.right, rc.bottom, TRUE);
        pos.MoveWindow(GetDlgItem(hWnd, IDC_DOWNLOAD_SETUP_GROUP), margin.cx, margin.cy, LOWORD(lParam) - (2 * margin.cx), rc.top - (2 * margin.cy), TRUE);
    }

    return(TRUE);
}

INT_PTR
CDownloadSetup::OnWmApp(HWND, WPARAM wParam, LPARAM)
{
    if(wParam == WM_DPICHANGED_BEFOREPARENT)
        ++m_dpiChanging;
    else if(wParam == WM_DPICHANGED_AFTERPARENT)
        --m_dpiChanging;

    return(TRUE);
}

INT_PTR
CDownloadSetup::OnWmDpiChanged(HWND, WPARAM wParam, LPARAM)
{
    const UINT dpiX = LOWORD(wParam);
    const UINT dpiY = HIWORD(wParam);
    m_ptMinTrackSize.x = Scale(m_ptMinTrackSize.x, m_dpiX, dpiX);
    m_ptMinTrackSize.y = Scale(m_ptMinTrackSize.y, m_dpiY, dpiY);
    m_dpiX = dpiX;
    m_dpiY = dpiY;
    return(TRUE);
}

INT_PTR CALLBACK
CDownloadSetup::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    static const DLG_MSGMAP<CDownloadSetup> sc_map[] =
    {
        { WM_SIZE,          &CDownloadSetup::OnWmSize           },
        { WM_GETMINMAXINFO, &CDownloadSetup::OnWmGetMinMaxInfo  },
        { WM_COMMAND,       &CDownloadSetup::OnWmCommand        },
        { WM_APP,           &CDownloadSetup::OnWmApp            },
        { WM_DPICHANGED,    &CDownloadSetup::OnWmDpiChanged     }
    };

    for(uintptr_t n = 0; n < _countof(sc_map); ++n)
    {
        if(sc_map[n].uMsg == uMsg)
        {
            CDownloadSetup* pThis = reinterpret_cast<CDownloadSetup*>(GetWindowLongPtr(hWnd, DWLP_USER));

            if(pThis)
                return((pThis->*sc_map[n].pfn)(hWnd, wParam, lParam));
        }
    }

    if(uMsg == WM_INITDIALOG)
    {
        SetWindowLongPtr(hWnd, DWLP_USER, lParam);
        return(reinterpret_cast<CDownloadSetup*>(lParam)->OnWmInitDialog(hWnd, wParam, lParam));
    }

    return(FALSE);
}
