#include "precomp.h"

CEnumConnections::CEnumConnections(CConnectionPoint* pItf)
    : m_pItf(pItf), m_lCursor(0), m_lRefCount(1)
{
    m_pItf->AddRef();
}

CEnumConnections::~CEnumConnections()
{
    m_pItf->Release();
}

IFACEMETHODIMP
CEnumConnections::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = static_cast<IUnknown*>(this);
        else if(IsEqualIID(riid, IID_IEnumConnections))
            *ppvObject = static_cast<IEnumConnections*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CEnumConnections::AddRef()
{
    return(InterlockedIncrement(&m_lRefCount));
}

IFACEMETHODIMP_(ULONG)
CEnumConnections::Release()
{
    const ULONG ulResult = InterlockedDecrement(&m_lRefCount);

    if(!ulResult)
        delete this;

    return(ulResult);
}

IFACEMETHODIMP
CEnumConnections::Next(ULONG cConnections, LPCONNECTDATA rgcd, ULONG* pcFetched)
{
    ULONG   lCursor;
    ULONG   ulTemp;
    BOOL    bl = IsBadWritePtrT(pcFetched);

    if(!bl)
        *pcFetched = 0;

#ifdef _DEBUG
    if(IsBadWritePtr(rgcd, (cConnections * sizeof(*rgcd))))
#else
    if(!rgcd)
#endif
        return(E_POINTER);

    ZeroMemory(rgcd, (cConnections * sizeof(*rgcd)));

    if(bl)
    {
        if((cConnections > 1) || (pcFetched))
            return(E_INVALIDARG);

        ulTemp = 0;
        pcFetched = &ulTemp;
    }

    for(bl = FALSE; ((*pcFetched < cConnections) && (!bl)); ++(*pcFetched))
    {
        do
        {
            if(rgcd[*pcFetched].pUnk)
            {
                rgcd[*pcFetched].pUnk->Release();
                rgcd[*pcFetched].pUnk = nullptr;
                rgcd[*pcFetched].dwCookie = 0;
            }

#ifndef _MSC_VER
            lCursor = InterlockedCompareExchangeAcquire(&m_lCursor, SIZE_MAX, SIZE_MAX);
#else
            // Microsoft uses Acquire semantics on volatile variables.
            lCursor = m_lCursor;
#endif

            if(S_OK != m_pItf->GetConnectData(lCursor, &rgcd[*pcFetched]))
            {
                --(*pcFetched);
                bl = TRUE;
                break;
            }

        } while(InterlockedCompareExchangeRelease(&m_lCursor, lCursor + 1, lCursor) != lCursor);
    }

    return((cConnections == *pcFetched) ? S_OK : S_FALSE);
}

IFACEMETHODIMP
CEnumConnections::Skip(ULONG cConnections)
{
    CONNECTDATA conn;
    ULONG       lCursor;

    while(cConnections--)
    {
        do
        {
#ifndef _MSC_VER
            lCursor = InterlockedCompareExchangeAcquire(&m_lCursor, SIZE_MAX, SIZE_MAX);
#else
            // Microsoft uses Acquire semantics on volatile variables.
            lCursor = m_lCursor;
#endif

            if(FAILED(m_pItf->GetConnectData(lCursor, &conn)))
                return(S_FALSE);

            conn.pUnk->Release();

        } while(InterlockedCompareExchangeRelease(&m_lCursor, lCursor + 1, lCursor) != lCursor);
    }

    return(S_OK);
}

IFACEMETHODIMP
CEnumConnections::Reset()
{
    InterlockedExchange(&m_lCursor, 0);
    return(S_OK);
}

IFACEMETHODIMP
CEnumConnections::Clone(IEnumConnections** ppEnum)
{
    CEnumConnections* pEnum;

    if(IsBadWritePtrT(ppEnum))
        return(E_POINTER);

    *ppEnum = nullptr;

    if(nullptr == (pEnum = new (std::nothrow) CEnumConnections(m_pItf)))
        return(E_OUTOFMEMORY);

#ifndef _MSC_VER
    pEnum->m_lCursor = InterlockedCompareExchangeAcquire(&m_lCursor, SIZE_MAX, SIZE_MAX);
#else
    // Microsoft uses Acquire semantics on volatile variables.
    pEnum->m_lCursor = m_lCursor;
#endif

    const HRESULT hr = pEnum->QueryInterface(IID_IEnumConnections, reinterpret_cast<void**>(ppEnum));
    pEnum->Release();
    return(hr);
}
