#include "precomp.h"

// For more information on Hex Merge Files, have a look at the example.hem
// that gets installed with TI's SmartRF FLASH Programmer.

HRESULT
CHexMergeFile::Load(LPCWSTR filename, SAFEARRAY* psa)
{
    static const struct
    {
        const char* pCmd;
        HRESULT (* fnHandler)(char* pParameters, SAFEARRAY* psa);
    } sc_cmdMap [] =
    {
        { "F", CHexMergeFile::cmdLoadFile       },
        { "A", CHexMergeFile::cmdSetSingleByte  }
    };

    HRESULT hr = S_OK;
    FILE*   file;
    char*   pParameters;
    char    line[2048];

    CCrtLocale locale(LC_ALL, TEXT("C"));
    const errno_t err = _wfopen_s(&file, filename, L"r");

    if(err)
    {
        if((err == EACCES) && (GetLastError() == ERROR_SHARING_VIOLATION))
            return(__HRESULT_FROM_WIN32(ERROR_SHARING_VIOLATION));

        errno = err;
        return(HresultFromErrno());
    }

    if(!file)
        return(E_UNEXPECTED);

    do
    {
        if(!fgets(line, _countof(line), file))
        {
            if(feof(file))
                break;

            hr = HresultFromErrno();
        }
        else
        {
            TrimA(line);

            if(IsComment(line) || (line[0] == '\0'))
                continue;

            if(nullptr == (pParameters = strchr(line, '=')))
            {
                hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
            }
            else
            {
                *pParameters++ ='\0';
                TrimA(line);
                hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);

                for(uintptr_t idx = 0; idx < _countof(sc_cmdMap); ++idx)
                {
                    if(!strcmp(sc_cmdMap[idx].pCmd, line))
                    {
                        TrimA(pParameters);
                        hr = sc_cmdMap[idx].fnHandler(pParameters, psa);
                        break;
                    }
                }
            }
        }
    } while(SUCCEEDED(hr));

    fclose(file);
    return(hr);
}

HRESULT
CHexMergeFile::cmdLoadFile(char* pName, SAFEARRAY* psa)
{
    BSTR    bstr;
    HRESULT hr = BSTRFromStringA(pName, &bstr);

    if(SUCCEEDED(hr))
    {
        hr = CIntelHexFile::Load(bstr, psa);
        SysFreeString(bstr);
    }

    return(hr);
}

HRESULT
CHexMergeFile::cmdSetSingleByte(char* pParameter, SAFEARRAY* psa)
{
    char* pData = strchr(pParameter, ',');
    char* endptr;
    long address;
    unsigned long data;

    if(!pData)
        return(__HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    *pData++ = '\0';
    TrimA(pParameter);
    address = strtol(pParameter, &endptr, 16);

    if((*endptr) || (address < 0) || (address == LONG_MAX) || (nullptr == (pParameter = strchr(pData, '='))))
        return(__HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    *pParameter++ = '\0';
    TrimA(pData);

    if(strcmp(pData, "D"))
        return(__HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    TrimA(pParameter);
    data = strtoul(pParameter, &endptr, 16);

    if((*endptr) || (data > 0xFF))
        return(__HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    return(AddBytes(psa, reinterpret_cast<const BYTE*>(&data), address, 1));
}

HRESULT
CHexMergeFile::AddBytes(SAFEARRAY* psa, const BYTE* pBlob, long lStartAddress, ULONG len)
{
    HRESULT hr = RedimAndLockArray(psa, lStartAddress, len);

    if(SUCCEEDED(hr))
    {
        void* pDst;

        if(SUCCEEDED(hr = SafeArrayPtrOfIndex(psa, &lStartAddress, &pDst)))
            CopyMemory(pDst, pBlob, len);

        SafeArrayUnlock(psa);
    }

    return(hr);
}
