#include "precomp.h"

HRESULT
CIntelHexFile::Load(LPCWSTR filename, SAFEARRAY* psa)
{
    HRESULT hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
    RECORD_INFO ri = {};
    FILE* file;
    char line[528];
    char* psz;
    unsigned long lExtendedAddress = 0;
    unsigned remainder = 0, eaten;
    long tmp;
    bool blIsSegmented = false, blEof = false;
    BYTE bCheckSum;
    unsigned cnt;
    BYTE* pData;
    size_t len;

    CCrtLocale locale(LC_ALL, TEXT("C"));
    const errno_t err = _wfopen_s(&file, filename, L"r");

    if(err)
    {
        if((err == EACCES) && (GetLastError() == ERROR_SHARING_VIOLATION))
            return(__HRESULT_FROM_WIN32(ERROR_SHARING_VIOLATION));

        errno = err;
        return(HresultFromErrno());
    }

    if(!file)
        return(E_UNEXPECTED);

    while(!feof(file))
    {
        if(!fgets(line, _countof(line), file))
        {
            if(!feof(file))
                hr = HresultFromErrno();

            break;
        }

        if(nullptr != (psz = strchr(line, '\n')))
            *psz = '\0';

        if(FAILED(hr = GetRecordInfo(line, &ri)))
            break;

        len = strlen(line);

        if(len < ((ri.len * 2) + 11))
        {
            hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
            break;
        }

        len -= 9;
        bCheckSum = LOBYTE(static_cast<unsigned>(ri.typ) + LOBYTE(ri.offset) + HIBYTE(ri.offset) + ri.len);

        switch(ri.typ)
        {
            case Data:
                eaten = 0;

                do
                {
                    if(blIsSegmented)
                    {
                        tmp = static_cast<long>((static_cast<unsigned long>(lExtendedAddress) << 4) + ri.offset);
                        remainder = ri.offset + ri.len;
                        remainder = (remainder > UINT16_MAX) ? LOWORD(remainder) : 0U;
                        ri.len -= remainder;
                    }
                    else
                    {
                        tmp = static_cast<long>((static_cast<unsigned long>(lExtendedAddress) << 16) + ri.offset);
                    }

                    if(SUCCEEDED(hr = RedimAndLockArray(psa, tmp, ri.len)))
                    {
                        if(SUCCEEDED(hr = SafeArrayPtrOfIndex(psa, &tmp, reinterpret_cast<void**>(&pData))))
                        {
                            for(cnt = 0; cnt < ri.len; ++cnt)
                            {
                                if(1 != _snscanf_s(&line[((cnt + eaten) * 2) + 9], len - ((cnt + eaten) * 2), "%2lx", &tmp))
                                {
                                    hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
                                    break;
                                }
                                *pData++ = LOBYTE(tmp);
                                bCheckSum += LOBYTE(tmp);
                            }
                        }
                        SafeArrayUnlock(psa);
                    }

                    eaten += ri.len;
                    ri.offset = 0;
                    ri.len = remainder;
                } while(remainder);

                ri.len = eaten;
                break;
            case EndOfFile:
                if(ri.len || ri.offset)
                    hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);

                blEof = true;
                break;
            case ExtendedSegmentAddress:
                if((1 != _snscanf_s(&line[9], len, "%4lx", &lExtendedAddress)) || (ri.len != 2) || (ri.offset))
                    hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
                else
                    blIsSegmented = true;

                bCheckSum += LOBYTE(LOBYTE(lExtendedAddress) + HIBYTE(lExtendedAddress));
                break;
            case ExtendedLinearAddress:
                if((1 != _snscanf_s(&line[9], len, "%4lx", &lExtendedAddress)) || (ri.len != 2) || (ri.offset))
                    hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
                else if(lExtendedAddress > INT16_MAX)
                    hr = SMARTICE04_E_INVALID_EXTENDEDLINEARADDRESS;
                else
                    blIsSegmented = false;

                remainder = 0;
                bCheckSum += LOBYTE(LOWORD(lExtendedAddress) + HIBYTE(LOWORD(lExtendedAddress)));
                break;
            case StartSegmentAddress:
            case StartLinearAddress:
                if((1 != _snscanf_s(&line[9], len, "%8lx", &tmp)) || (ri.len != 4) || (ri.offset))
                    hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);

                for(cnt = 0; cnt < 4; ++cnt)
                {
                    bCheckSum += LOBYTE(tmp);
                    tmp /= 256;
                }

                break;
            default:
                break;
        }

        if(1 != _snscanf_s(&line[(ri.len * 2) + 9], len - (ri.len * 2), "%2lx", &tmp))
        {
            if(SUCCEEDED(hr))
                hr = __HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
        }
        else if(static_cast<BYTE>(tmp + bCheckSum))
        {
            if(SUCCEEDED(hr))
                hr = __HRESULT_FROM_WIN32(ERROR_CRC);
        }

        if(blEof || FAILED(hr))
            break;
    }

    fclose(file);
    return(hr);
}

HRESULT
CIntelHexFile::GetRecordInfo(const char* line, RECORD_INFO* pri)
{
    unsigned typ;

    if(*line != ':')
        return(__HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    typ = MAKEWORD(line[8], line[7]) - 0x3030U;

    if(typ > static_cast<unsigned int>(StartLinearAddress))
        return(__HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    if(sscanf_s(&line[1], "%2x%4x", &pri->len, &pri->offset) < 2)
        return(__HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    pri->typ = static_cast<RecordType>(typ);
    return(S_OK);
}
