#include "precomp.h"

IFACEMETHODIMP
CSmartICE04EventsSink::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_ISmartICE04Events))
            *ppvObject = static_cast<ISmartICE04Events*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CSmartICE04EventsSink::AddRef()
{
    return(2);
}

IFACEMETHODIMP_(ULONG)
CSmartICE04EventsSink::Release()
{
    return(1);
}

IFACEMETHODIMP
CSmartICE04EventsSink::OnTargetVoltageChanged(IDispatch* sender, VARIANT_BOOL)
{
    ISmartICE04* pDevice = nullptr;

    if(SUCCEEDED(sender->QueryInterface(IID_ISmartICE04, reinterpret_cast<void**>(&pDevice))))
    {
        TCHAR sz[32];
        long voltage = 0;

        pDevice->get_TargetVoltage(&voltage);
        StringCchPrintf(sz, _countof(sz), TEXT("%ld mV"), voltage);
        m_DeviceList.SetTargetVoltage(pDevice, sz);
        pDevice->Release();
    }
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04EventsSink::OnTargetChanged(IDispatch* sender, ChipId chipId)
{
    ISmartICE04* pDevice = nullptr;
    ITypeInfo* pTypeInfo = nullptr;
    TYPEATTR* pAttr = nullptr;
    VARIANT var;
    UINT cnt;

    if(FAILED(sender->QueryInterface(IID_ISmartICE04, reinterpret_cast<void**>(&pDevice))))
        return(S_OK);

    if(chipId == UnknownChip)
    {
        TCHAR sz[32];

        if(LoadString(g_hInst, IDS_NONE, sz, _countof(sz)))
        {
            m_DeviceList.SetTarget(pDevice, sz);
            pDevice->Release();
            return(S_OK);
        }
    }

    if(SUCCEEDED(::GetTypeInfoOfChipIdEnumeration(pDevice, &pTypeInfo)))
    {
        if(SUCCEEDED(pTypeInfo->GetTypeAttr(&pAttr)))
        {
            if(pAttr->typekind == TKIND_ENUM)
            {
                VariantInit(&var);

                for(UINT idx = 0; idx < pAttr->cVars; ++idx)
                {
                    VARDESC* pDesc = nullptr;

                    if(SUCCEEDED(pTypeInfo->GetVarDesc(idx, &pDesc)))
                    {
                        if(SUCCEEDED(VariantChangeTypeEx(&var, pDesc->lpvarValue, CThreadLocale::getLCID(), 0, VT_I4)))
                        {
                            if(V_I4(&var) == static_cast<long>(chipId))
                            {
                                BSTR bstr = nullptr;
                                if(SUCCEEDED(pTypeInfo->GetNames(pDesc->memid, &bstr, 1, &cnt)))
                                {
                                    m_DeviceList.SetTarget(pDevice, bstr);
                                    SysFreeString(bstr);
                                }
                            }
                        }
                        pTypeInfo->ReleaseVarDesc(pDesc);
                        VariantClear(&var);
                    }
                }
            }
            pTypeInfo->ReleaseTypeAttr(pAttr);
        }
        pTypeInfo->Release();
    }

    pDevice->Release();
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04EventsSink::OnDebugModeChanged(IDispatch* sender, VARIANT_BOOL isInDebugMode)
{
    ISmartICE04* pDevice = nullptr;
    if(SUCCEEDED(sender->QueryInterface(IID_ISmartICE04, reinterpret_cast<void**>(&pDevice))))
    {
        TCHAR sz[32];

        if(LoadString(g_hInst, static_cast<UINT>((isInDebugMode) ? IDS_DEBUGMODE : IDS_IDLE), sz, _countof(sz)))
            m_DeviceList.SetDebugMode(pDevice, sz);

        pDevice->Release();
    }
    return(S_OK);
}

HRESULT
CSmartICE04EventsSink::GetDevice(ISmartICE04** ppDevice)
{
    *ppDevice = nullptr;

    if(!m_pCP)
        return(OLE_E_BLANK);

    IConnectionPointContainer* pCPC = nullptr;
    HRESULT hr = m_pCP->GetConnectionPointContainer(&pCPC);

    if(SUCCEEDED(hr))
    {
        hr = pCPC->QueryInterface(IID_ISmartICE04, reinterpret_cast<void**>(ppDevice));
        pCPC->Release();
    }

    return(hr);
}
