#include "precomp.h"

const TCHAR CSmartICE04Progress::m_szSmartICE04Progress[] = TEXT("E1D20737-7D34-4868-A671-2033AAC3FDFE");

CSmartICE04Progress::CSmartICE04Progress(ITypeLib* pTlb, ISmartICE04ProgressDialog* pDlg, HWND hWndParent, HRESULT* phr)
    : m_hWnd(nullptr), m_pTaskbarList(nullptr), m_pTypeInfo(nullptr), m_pDispTypeInfo(nullptr), m_wndproc(nullptr), m_hWndParent(hWndParent), m_lRefCount(1), m_uFlags(0)
{
    LCID lcid = CThreadLocale::getLCID();

    ::LockServer();
    pDlg->get_LocaleID(&lcid);

    if(m_hWndParent)
    {
        if(!IsWindow(m_hWndParent))
        {
            *phr = E_INVALIDARG;
        }
        else
        {
            HWND hwndAncestor = GetAncestor(m_hWndParent, GA_ROOT);

            if(hwndAncestor)
            {
                m_uFlags = (IsWindowEnabled(hwndAncestor)) ? FLAG_PARENTENABLED : 0U;

                if(!GetProp(hwndAncestor, m_szSmartICE04Progress))
                {
                    if(SetProp(hwndAncestor, m_szSmartICE04Progress, reinterpret_cast<HANDLE>(this)))
                    {
                        m_uFlags |= FLAG_PROPERTYVALID;
                        m_wndproc = reinterpret_cast<WNDPROC>(SetWindowLongPtr(hwndAncestor, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(SubClassProc)));
                    }
                }
            }
        }
    }

    if(SUCCEEDED(*phr))
    {
        INITCOMMONCONTROLSEX iccex;
        iccex.dwSize = sizeof(iccex);
        iccex.dwICC = ICC_PROGRESS_CLASS;
        InitCommonControlsEx(&iccex);

        HANDLE hThread = CreateThread(nullptr, 0, ThreadProc, this, 0, nullptr);

        if(!hThread)
        {
            *phr = HRESULT_FROM_WIN32(GetLastError());
        }
        else
        {
            DWORD dwExitCode;

            while(!IsWindow(m_hWnd))
            {
                if(WAIT_TIMEOUT == WaitForSingleObject(hThread, 1))
                    continue;

                GetExitCodeThread(hThread, &dwExitCode);
                *phr = HRESULT_FROM_WIN32(dwExitCode);
                break;
            }

            CloseHandle(hThread);
        }

        if(m_hWnd)
        {
            CThreadLocale   loc(MAKELCID(MAKELANGID(PRIMARYLANGID(lcid), SUBLANG_DEFAULT), SORT_DEFAULT));
            TCHAR           szText[48];

            if(FormatMessage(FORMAT_MESSAGE_FROM_HMODULE | FORMAT_MESSAGE_IGNORE_INSERTS, g_hInst, MSG_CANCEL, 0, szText, _countof(szText), nullptr))
            {
                SetDlgItemText(m_hWnd, IDCANCEL, szText);
            }

            BSTR bstr = nullptr;
            if(SUCCEEDED(*phr = pDlg->get_Line1(&bstr)))
            {
                SetDlgItemTextW(m_hWnd, IDC_LINE1, (bstr) ? bstr : L"");
                SysFreeString(bstr);
                bstr = nullptr;

                if(SUCCEEDED(*phr = pDlg->get_Line2(&bstr)))
                {
                    SetDlgItemTextW(m_hWnd, IDC_LINE2, (bstr) ? bstr : L"");
                    SysFreeString(bstr);
                    bstr = nullptr;

                    if(SUCCEEDED(*phr = pDlg->get_Title(&bstr)))
                    {
                        SetWindowTextW(m_hWnd, (bstr) ? bstr : L"");
                        SysFreeString(bstr);
                    }
                }
            }
        }
        else if(SUCCEEDED(*phr))
        {
            *phr = E_UNEXPECTED;
        }
    }

    if(SUCCEEDED(*phr))
    {
        if(SUCCEEDED(*phr = pTlb->GetTypeInfoOfGuid(IID_ISmartICE04Progress, &m_pTypeInfo)))
        {
            if(SUCCEEDED(*phr = pTlb->GetTypeInfoOfGuid(DIID_ISmartICE04ProgressDisp, &m_pDispTypeInfo)))
            {
                if(FAILED(CoCreateInstance(CLSID_TaskbarList, nullptr, CLSCTX_SERVER, __uuidof(m_pTaskbarList), reinterpret_cast<void**>(&m_pTaskbarList))))
                {
                    m_pTaskbarList = nullptr;
                }
            }
        }
    }
}

CSmartICE04Progress::~CSmartICE04Progress()
{
    HWND hwndAncestor = GetAncestor(m_hWndParent, GA_ROOT);

    if(m_uFlags & FLAG_PARENTENABLED)
    {
        if(hwndAncestor)
            EnableWindow(hwndAncestor, TRUE);
    }

    if(IsWindow(m_hWnd))
    {
        if(m_pTaskbarList)
            m_pTaskbarList->SetProgressState(m_hWnd, TBPF_NOPROGRESS);

        PostMessage(m_hWnd, WM_CLOSE, 0, 0);
    }

    if(m_uFlags & FLAG_PROPERTYVALID)
    {
        if(hwndAncestor)
        {
            if(m_wndproc)
                SetWindowLongPtr(hwndAncestor, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(m_wndproc));

            RemoveProp(hwndAncestor, m_szSmartICE04Progress);
        }
    }

    if(m_pTaskbarList)
        m_pTaskbarList->Release();
    if(m_pTypeInfo)
        m_pTypeInfo->Release();
    if(m_pDispTypeInfo)
        m_pDispTypeInfo->Release();
    if(IsWindow(m_hWnd))
        PostMessage(m_hWnd, WM_CLOSE, 0, 0);

    const DWORD dwTicks = GetTickCount();

    while(IsWindow(m_hWnd))
    {
        while((GetTickCount() - dwTicks) > 150)
            break;

        SwitchToThread();
    }

    ::UnlockServer();
}

IFACEMETHODIMP
CSmartICE04Progress::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IDispatch))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, DIID_ISmartICE04ProgressDisp))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, IID_ISmartICE04Progress))
            *ppvObject = static_cast<ISmartICE04Progress*>(this);
        else if(IsEqualIID(riid, IID_ISupportErrorInfo))
            *ppvObject = static_cast<ISupportErrorInfo*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CSmartICE04Progress::AddRef()
{
    return(InterlockedIncrement(&m_lRefCount));
}

IFACEMETHODIMP_(ULONG)
CSmartICE04Progress::Release()
{
    const ULONG ulResult = InterlockedDecrement(&m_lRefCount);

    if(!ulResult)
        delete this;

    return(ulResult);
}

IFACEMETHODIMP
CSmartICE04Progress::GetTypeInfoCount(UINT* pctinfo)
{
    if(IsBadWritePtrT(pctinfo))
        return(E_POINTER);

    *pctinfo = 1;
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04Progress::GetTypeInfo(UINT iTInfo, LCID, ITypeInfo** ppTInfo)
{
    if(IsBadWritePtrT(ppTInfo))
        return(E_POINTER);

    if(iTInfo)
    {
        *ppTInfo = nullptr;
        return(DISP_E_BADINDEX);
    }

    *ppTInfo = m_pDispTypeInfo;
    (*ppTInfo)->AddRef();
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04Progress::GetIDsOfNames(REFIID, LPOLESTR* rgszNames, UINT cNames, LCID, DISPID* rgDispId)
{
    return(m_pDispTypeInfo->GetIDsOfNames(rgszNames, cNames, rgDispId));
}

IFACEMETHODIMP
CSmartICE04Progress::Invoke(DISPID dispIdMember, REFIID, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    CThreadLocale   loc(lcid);
    UINT            uArgErr;
    return(m_pTypeInfo->Invoke(static_cast<ISmartICE04Progress*>(this), dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, (puArgErr) ? puArgErr : &uArgErr));
}

IFACEMETHODIMP
CSmartICE04Progress::Start()
{
    if(!IsWindow(m_hWnd))
        return(RaiseCOMException(DISPID_SMARTICE04_START, E_UNEXPECTED));

    DisableParent();
    EnableDlgItem(m_hWnd, IDCANCEL, TRUE);
    SendDlgItemMessage(m_hWnd, IDC_PROGRESSBAR, PBM_SETRANGE32, 0, 100);
    SendDlgItemMessage(m_hWnd, IDC_PROGRESSBAR, PBM_SETPOS, 0, 0);
    ShowWindow(m_hWnd, SW_SHOW);

    if(m_pTaskbarList)
    {
        m_pTaskbarList->SetProgressState(m_hWnd, TBPF_NORMAL);
        m_pTaskbarList->SetProgressValue(m_hWnd, 0ULL, 100ULL);
    }

    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04Progress::Stop()
{
    EnableParent();

    if(m_pTaskbarList)
        m_pTaskbarList->SetProgressState(m_hWnd, TBPF_NOPROGRESS);

    if(!IsWindow(m_hWnd))
        return(RaiseCOMException(DISPID_SMARTICE04_STOP, E_UNEXPECTED));

    ShowWindow(m_hWnd, SW_HIDE);
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04Progress::SetProgress(long lCompleted, long lTotal)
{
    if(!IsWindow(m_hWnd))
        return(RaiseCOMException(DISPID_SMARTICE04_SETPROGRESS, E_UNEXPECTED));

    if(m_pTaskbarList)
        m_pTaskbarList->SetProgressValue(m_hWnd, static_cast<unsigned long>(lCompleted), static_cast<unsigned long>(lTotal));

    if(IsWindowVisible(m_hWnd))
    {
        MSG msg;

        while(PeekMessage(&msg, nullptr, 0, 0, PM_REMOVE))
        {
            TranslateMessage(&msg);
            DispatchMessage (&msg);
        }
    }

    HWND hWnd = GetDlgItem(m_hWnd, IDC_PROGRESSBAR);
    PostMessage(hWnd, PBM_SETRANGE32, 0, lTotal);
    PostMessage(hWnd, PBM_SETPOS, static_cast<WPARAM>(lCompleted), 0);
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04Progress::get_Canceled(VARIANT_BOOL* pblCanceled)
{
    if(IsBadWritePtrT(pblCanceled))
        return(RaiseCOMException(DISPID_SMARTICE04_CANCELED, E_POINTER));

    if(!IsWindow(m_hWnd))
    {
        *pblCanceled = VARIANT_FALSE;
        return(RaiseCOMException(DISPID_SMARTICE04_CANCELED, E_UNEXPECTED));
    }

    if(IsWindowVisible(m_hWnd))
    {
        MSG msg;

        while(PeekMessage(&msg, nullptr, 0, 0, PM_REMOVE))
        {
            TranslateMessage(&msg);
            DispatchMessage (&msg);
        }
    }

    *pblCanceled = (IsWindowEnabled(GetDlgItem(m_hWnd, IDCANCEL))) ? VARIANT_FALSE : VARIANT_TRUE;
    return(S_OK);
}

IFACEMETHODIMP
CSmartICE04Progress::InterfaceSupportsErrorInfo(REFIID riid)
{
    __try
    {
        if(IsEqualIID(riid, DIID_ISmartICE04ProgressDisp))
            return(S_OK);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(S_FALSE);
}

void
CSmartICE04Progress::EnableParent() const
{
    if(m_uFlags & FLAG_PARENTENABLED)
    {
        HWND hWnd = GetAncestor(m_hWndParent, GA_ROOT);

        if(hWnd)
            EnableWindow(hWnd, TRUE);
    }
}

void
CSmartICE04Progress::DisableParent() const
{
    if(m_uFlags & FLAG_PARENTENABLED)
    {
        HWND hWnd = GetAncestor(m_hWndParent, GA_ROOT);

        if(hWnd)
            EnableWindow(hWnd, FALSE);
    }
}

DWORD WINAPI
CSmartICE04Progress::ThreadProc(LPVOID lpParameter)
{
    CThreadDpiAwarenessContext  dpiAwarness(DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2);
    CSmartICE04Progress*        pProgress = static_cast<CSmartICE04Progress*>(lpParameter);
    MSG                         msg;

    if(nullptr == (pProgress->m_hWnd = CreateDialogParam(g_hInst, MAKEINTRESOURCE(IDD_PROGRESS), nullptr, DlgProc, reinterpret_cast<LPARAM>(pProgress))))
        return(static_cast<DWORD>(HRESULT_FROM_WIN32(GetLastError())));

    CenterWindow(pProgress->m_hWnd, pProgress->m_hWndParent);

    while(GetMessage(&msg, nullptr, 0, 0))
    {
        if(!IsDialogMessage(pProgress->m_hWnd, &msg))
        {
            TranslateMessage(&msg);
            DispatchMessage (&msg);
        }
    }

    return(0);
}

void
CSmartICE04Progress::PreventPinning(HWND hWnd, BOOL enable)
{
    IPropertyStore* pPropStore = nullptr;
    PROPVARIANT var;

    if(SUCCEEDED(SHGetPropertyStoreForWindow(hWnd, IID_IPropertyStore, reinterpret_cast<void**>(&pPropStore))))
    {
        if(pPropStore)
        {
            if(enable)
            {
                if(SUCCEEDED(InitPropVariantFromBoolean(TRUE, &var)))
                {
                    pPropStore->SetValue(PKEY_AppUserModel_PreventPinning, var);
                    PropVariantClear(&var);
                }

                if(SUCCEEDED(InitPropVariantFromString(L"TBK.SmartICE04.Progress", &var)))
                {
                    pPropStore->SetValue(PKEY_AppUserModel_ID, var);
                    PropVariantClear(&var);
                }
            }
            else
            {
                PropVariantInit(&var);
                pPropStore->SetValue(PKEY_AppUserModel_ID, var);
                pPropStore->SetValue(PKEY_AppUserModel_PreventPinning, var);
            }

            pPropStore->Release();
        }
    }
}

INT_PTR CALLBACK
CSmartICE04Progress::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM)
{
    switch(uMsg)
    {
        case WM_INITDIALOG:
            CSmartICE04Progress::PreventPinning(hWnd, TRUE);
            return(TRUE);
        case WM_COMMAND:
            if(LOWORD(wParam) == IDCANCEL)
            {
                EnableDlgItem(hWnd, IDCANCEL, FALSE);
                return(TRUE);
            }
            break;
        case WM_SYSCOMMAND:
            if(wParam == SC_CLOSE)
            {
                EnableDlgItem(hWnd, IDCANCEL, FALSE);
                return(TRUE);
            }
            break;
        case WM_CLOSE:
            DestroyWindow(hWnd);
            return(TRUE);
        case WM_DESTROY:
            CSmartICE04Progress::PreventPinning(hWnd, FALSE);
            break;
        case WM_NCDESTROY:
            PostQuitMessage(0);
            break;
        default:
            break;
    }

    return(FALSE);
}

LRESULT CALLBACK
CSmartICE04Progress::SubClassProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    CSmartICE04Progress* pThis = reinterpret_cast<CSmartICE04Progress*>(GetProp(hWnd, m_szSmartICE04Progress));

    if(!pThis)
        return(DefWindowProc(hWnd, uMsg, wParam, lParam));

    if(IsWindowVisible(pThis->m_hWnd))
    {
        if((uMsg == WM_ACTIVATE) && (wParam != WA_INACTIVE))
        {
            SetForegroundWindow(pThis->m_hWnd);
        }
        else if((uMsg == WM_ACTIVATEAPP) && (wParam))
        {
            SetForegroundWindow(pThis->m_hWnd);
        }
    }

    return(CallWindowProc(pThis->m_wndproc, hWnd, uMsg, wParam, lParam));
}
