#include "precomp.h"

HRESULT RaiseCOMException(IDispatch* pDispatch, DISPID dispid, HRESULT hr)
{
    IProvideClassInfo*  pPCI        = nullptr;
    ICreateErrorInfo*   pCreateErr  = nullptr;
    IErrorInfo*         pError      = nullptr;
    ITypeInfo*          pInfo       = nullptr;
    TYPEATTR*           ptattr      = nullptr;
    LPOLESTR            pSource     = nullptr;
    DWORD               dwHelpContext;
    DWORD               dwMessageLen;
    TCHAR               szError[1024];
    BSTR                bstrHelpFile = nullptr;

    if((HRESULT_FACILITY(hr) == FACILITY_ITF) && (HRESULT_CODE(hr) >= 0x0200))
    {
        CThreadLocale loc(MAKELCID(MAKELANGID(PRIMARYLANGID(CThreadLocale::getLANGID()), SUBLANG_DEFAULT), SORT_DEFAULT));
        dwMessageLen = FormatMessage(FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_HMODULE, g_hInst, static_cast<DWORD>(hr), 0, szError, _countof(szError), nullptr);
    }
    else
    {
        dwMessageLen = FormatMessage(FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM, nullptr, static_cast<DWORD>(hr), 0, szError, _countof(szError), nullptr);
    }

    if(!dwMessageLen)
    {
        StringCchPrintf(szError, _countof(szError) - 1, TEXT("Unknown error: 0x%08lX"), hr);
    }

    if(SUCCEEDED(CreateErrorInfo(&pCreateErr)))
    {
        #ifdef UNICODE
          pCreateErr->SetDescription(szError);
        #else
          WCHAR wszError[_countof(szError)];
          MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED | MB_ERR_INVALID_CHARS, szError, -1, wszError, _countof(wszError));
          wszError[_countof(szError) - 1] = TEXT('\0');
          pCreateErr->SetDescription(wszError);
        #endif

        if(SUCCEEDED(pDispatch->QueryInterface(IID_IProvideClassInfo, reinterpret_cast<void**>(&pPCI))))
        {
            if(SUCCEEDED(pPCI->GetClassInfo(&pInfo)))
            {
                if(SUCCEEDED(pInfo->GetTypeAttr(&ptattr)))
                {
                    if(ptattr->wTypeFlags & TYPEFLAG_FCANCREATE)
                    {
                        if(SUCCEEDED(ProgIDFromCLSID(ptattr->guid, &pSource)))
                        {
                            pCreateErr->SetSource(pSource);
                            CoTaskMemFree(pSource);
                        }
                    }
                    pInfo->ReleaseTypeAttr(ptattr);
                    ptattr = nullptr;
                }
                pInfo->Release();
                pInfo = nullptr;
            }
            pPCI->Release();
        }

        if(SUCCEEDED(pDispatch->GetTypeInfo(0, CThreadLocale::getLCID(), &pInfo)))
        {
            if(SUCCEEDED(pInfo->GetTypeAttr(&ptattr)))
            {
                pCreateErr->SetGUID(ptattr->guid);
                pInfo->ReleaseTypeAttr(ptattr);
            }

            if(SUCCEEDED(pInfo->GetDocumentation(dispid, nullptr, nullptr, &dwHelpContext, &bstrHelpFile)))
            {
                pCreateErr->SetHelpContext(dwHelpContext);
                pCreateErr->SetHelpFile(bstrHelpFile);
                SysFreeString(bstrHelpFile);
            }

            pInfo->Release();
        }

        if(SUCCEEDED(pCreateErr->QueryInterface(IID_IErrorInfo, reinterpret_cast<void**>(&pError))))
        {
            SetErrorInfo(0, pError);
            pError->Release();
        }

        pCreateErr->Release();
    }

    return(hr);
}
