#include "precomp.h"

// The documentation for this function can be found at
// https://docs.microsoft.com/en-us/windows/win32/api/shlwapi/nc-shlwapi-dllgetversionproc
STDAPI DllGetVersion(DLLVERSIONINFO2* pdvi2)
{
    VS_FIXEDFILEINFO*   pvsffi = nullptr;
    HRESULT             hr = S_OK;
    DWORD               dwHandle = 0, dwSize;
    TCHAR               szFileName[MAX_PATH * 2];
    BYTE*               pMem;
    UINT                uLen;

    if((!pdvi2) || ((pdvi2->info1.cbSize != sizeof(*pdvi2)) && (pdvi2->info1.cbSize != sizeof(pdvi2->info1))))
        return(E_INVALIDARG);

    ZeroMemory(&pdvi2->info1.dwMajorVersion, (pdvi2->info1.cbSize - offsetof(DLLVERSIONINFO2, info1.dwMajorVersion)));
    szFileName[0] = TEXT('\0');

    if(!GetModuleFileName(g_hInst, szFileName, _countof(szFileName)))
        return(E_UNEXPECTED);
    if(0 == (dwSize = GetFileVersionInfoSize(szFileName, &dwHandle)))
        return(E_UNEXPECTED);
    if(nullptr == (pMem = new (std::nothrow) BYTE[dwSize]))
        return(E_OUTOFMEMORY);
#pragma warning(suppress: 6388)
    if(GetFileVersionInfo(szFileName, dwHandle, dwSize, pMem))
        VerQueryValue(pMem, TEXT("\\"), reinterpret_cast<PVOID*>(&pvsffi), &uLen);

    if(!pvsffi)
    {
        hr = HRESULT_FROM_WIN32(GetLastError());
    }
    else
    {
        pdvi2->info1.dwMajorVersion = HIWORD(pvsffi->dwFileVersionMS);
        pdvi2->info1.dwMinorVersion = LOWORD(pvsffi->dwFileVersionMS);
        pdvi2->info1.dwBuildNumber  = HIWORD(pvsffi->dwFileVersionLS);
        pdvi2->info1.dwPlatformID   = static_cast<DWORD>((pvsffi->dwFileOS == VOS__WINDOWS32) ? DLLVER_PLATFORM_WINDOWS : DLLVER_PLATFORM_NT);

        if(pdvi2->info1.cbSize >= sizeof(DLLVERSIONINFO2))
        {
            pdvi2->dwFlags    = 0;
            pdvi2->ullVersion = MAKEDLLVERULL(pdvi2->info1.dwMajorVersion, pdvi2->info1.dwMinorVersion, pdvi2->info1.dwBuildNumber, LOWORD(pvsffi->dwFileVersionLS));
        }
    }

    delete [] pMem;
    return(hr);
}
