#include "precomp.h"

struct dbgblk     *pdbg;      // startup values
struct MonConf  MonConf;      // holds target-setup values

/*
 * Analyze the command arguments 'pArg' and initialize 'MonConf'
 */

static void AnalyzeMonParams (char * /* pPath */, char* pArgs)  {
    char* pContext = nullptr;
    int tmp;
    char* ptmp =_strdup(pArgs);

    ZeroMemory(&MonConf, sizeof(MonConf));
    MonConf.Opt = FLASH_ERASE | FLASH_PROGRAM | FLASH_VERIFY | CACHE_CODE;

    for(char* pToken = strtok_s(((ptmp) ? ptmp : pArgs), "/", &pContext); pToken; pToken = strtok_s(nullptr, "/", &pContext))
    {
        switch(*pToken++)
        {
            case 'D':
                TrimA(pToken);

                if(strlen(pToken) < _countof(MonConf.idDevice))
                    StringCchCopyA(MonConf.idDevice, _countof(MonConf.idDevice), pToken);

                break;
            case 'O':                              // some Options
                if(StrToIntExA(pToken, STIF_DEFAULT, &tmp))
                    MonConf.Opt = static_cast<unsigned>(tmp);

                break;
            default:
                break;
        }
    }

    free(ptmp);
}

/*
 * Create an argument string out of current 'MonConf'.  This string
 * will be registered with the current project.
 * Note: the maximum length of the string must not exceed 'ValSize-1'
 *       characters. ValSize is a member of QDLL (in Bom.h). The current
 *       maximum is 1024 characters including the zero terminator.
 */

void WriteMonParams(char *pArgs, size_t cchMax)  {
  StringCchPrintfA(pArgs, cchMax, "/D%s /O%u", MonConf.idDevice, MonConf.Opt);
}


/*
 * Start the Configuration-Parameter Setup Dialog.
 * Note: This dialog *MUST* be modal, modeless is not allowed!
 */

static int DoDlgSetup (HWND hWndParent)  {
    return((S_OK == CTargetSetup::DoModal(hWndParent)) ? IDOK : IDCANCEL);
}

/*
 * Start the Configuration-Parameter Flash Download Setup Dialog.
 * Note: this dialog *MUST* be modal, modeless is not allowed !
 */

static int DoFDlgSetup (HWND hWndParent)  {
    return((S_OK == CDownloadSetup::DoModal(hWndParent)) ? IDOK : IDCANCEL);
}


#ifdef __cplusplus
 extern "C"  {  // must avoid C++ mangled names here!
#endif

/*
 * Checker Entry for Target-Dll loader
 *  Note: When the uVision2 debugger is started in target mode, this
 *        function is called first to check the validity of the DLL.
 *
 * The name of this function depends on architecture:
 *        80167: EnumUv3167()
 *        8051:  EnumUv351()
 *        80251: EnumUv3251()
 */

int AGDI_DLLEXPORT EnumUv351 (void *p, DWORD nCode)  {
  switch (nCode)  {
    case 0:
      break;
    case 1:          // not used
      break;
    case 2:          // register debug block
      pdbg = static_cast<struct dbgblk*>(p);  // defined in ComTyp.h
      AnalyzeMonParams (pdbg->TargDll, pdbg->TargArgs);
      return (8051);                     // Ok.
    default:
      break;
  }
  return (0);                            // Ok.
}

/*
 * This function is required for Remote-Setup by Target-Options-Debug
 * and Target-Options-Utilities Sheet.
 */

int AGDI_DLLEXPORT DllUv3Cap (DWORD nCode, void *p)  {
  QDLL    *pQ;
  int     i = 0;

  switch (nCode)  {
    case 2:                          // match family
      i = 8051;                      // identify as 8051 Target driver
      break;

    case 100:                        // has Flash-DownLoad Capabilities
      return (1);                    // say 'Yes'

    case 1:                          // Cpu/Target-DLL Settings
      pQ = static_cast<QDLL*>(p);    // refer to Bom.H for 'QDLL'
      AnalyzeMonParams (pQ->pathUv3, pQ->value);
      i = DoDlgSetup(pQ->parent);    // start Target Setup Dialog...
      if (i == IDOK)  {              // take the values...
        WriteMonParams (pQ->value, static_cast<size_t>(pQ->ValSize));   // create an ASCII string out of MonConf
        i = 1;
      }
      else i = 0;                    // don't register string in project.
      break;

    case 110:                        // Cpu/Target/Flash-DLL Settings
      pQ = static_cast<QDLL*>(p);
      AnalyzeMonParams (pQ->pathUv3, pQ->value);
      i = DoFDlgSetup(pQ->parent);   // Flash Download Setup Dialog...
      if (i == IDOK)  {
        WriteMonParams (pQ->value, static_cast<size_t>(pQ->ValSize));
        if (!(MonConf.Opt & (FLASH_ERASE | FLASH_PROGRAM | FLASH_VERIFY)))
        {
          CActCtx ctx(IDR_MANIFEST);
          MyTaskDialog(hMfrm, g_hInst, MAKEINTRESOURCE(IDS_SMARTICE04), MAKEINTRESOURCE(IDS_FLASHWARNING), MAKEINTRESOURCE(IDS_NOFUNC), TDCBF_OK_BUTTON, TD_WARNING_ICON, nullptr);
        }
        i = 1;
      }
      else i = 0;                    // don't register MonConf changes...
      break;
    default:
      break;
  }
  return (i);                        // return value
}


#ifdef __cplusplus
 }              // End 'extern "C"'
#endif
