#ifndef SELFREG_H
#define SELFREG_H

namespace SELFREG_DATA
{
    typedef enum
    {
        SELFREG_TYPE_NULL = 0,
        SELFREG_TYPE_ROOTKEY,
        SELFREG_TYPE_KEY,
        SELFREG_TYPE_VALUE_SZ,
        SELFREG_TYPE_VALUE_DWORD,
        SELFREG_TYPE_TYPELIB,
        SELFREG_TYPE_REGISTRY_MAP,
        SELFREG_TYPE_TYPELIB_MAP,
        SELFREG_TYPE_CALLBACK_MAP,
        SELFREG_TYPE_CALLBACK
    } SELFREG_TYPE_ENUM;

    typedef struct
    {
        SELFREG_TYPE_ENUM StructType;
        BOOL blRemoveable;
        PCTSTR Name;

        union
        {
            PCTSTR sz;
            DWORD   dw;
        } Value;
    } SELFREG_STRUCT_KEYORVALUE;

#pragma warning(push)
#pragma warning(disable: 4820)
    typedef struct
    {
        SELFREG_TYPE_ENUM StructType;
        unsigned short    wVerMajor;
        unsigned short    wVerMinor;
        const GUID*       pGuid;
        LCID              lcid;
    } SELFREG_STRUCT_TYPELIB;
#pragma warning(pop)

    typedef HRESULT (WINAPI * SELFREG_CALLBACKPROC)();

    typedef struct
    {
        SELFREG_TYPE_ENUM     StructType;
        BOOL                  blOnRegister;
        SELFREG_CALLBACKPROC  pfn;
    } SELFREG_STRUCT_CALLBACK;

    extern const void* SelfregMap[];
}

#define SELFREG_MAP_BEGIN               namespace SELFREG_DATA { const void* SelfregMap[] = {
#define SELFREG_MAP_END                 , nullptr };}

#define SELFREG_REGISTRY_BEGIN(name)    namespace SELFREG_DATA {  const SELFREG_STRUCT_KEYORVALUE (name)[] = {{ SELFREG_TYPE_REGISTRY_MAP, FALSE, nullptr, { nullptr }},
#define SELFREG_REGISTRY_END            { SELFREG_TYPE_NULL, FALSE, nullptr, { nullptr }}};}
#define SELFREG_ROOT_BEGIN(hkey)        { SELFREG_TYPE_ROOTKEY, FALSE,  (PCTSTR)(hkey), { nullptr }},
#define SELFREG_KEY_BEGIN(name)         { SELFREG_TYPE_KEY, TRUE, (name), { nullptr }},
#define SELFREG_KEY_BEGIN_PERSIST(name) { SELFREG_TYPE_KEY, FALSE, (name), { nullptr }},
#define SELFREG_VALUE_SZ(name, val)     { SELFREG_TYPE_VALUE_SZ, TRUE, (name), { (val) }},
#define SELFREG_VALUE_DW(name, val)     { SELFREG_TYPE_VALUE_DWORD, TRUE, (name), { (PCTSTR)(DWORD_PTR)(DWORD)(val) }},
#define SELFREG_END                     { SELFREG_TYPE_NULL, FALSE, nullptr, { nullptr }},

#define SELFREG_TYPELIB_BEGIN(name)     namespace SELFREG_DATA {  const SELFREG_STRUCT_TYPELIB (name)[] = {{ SELFREG_TYPE_TYPELIB_MAP, 0, 0, nullptr, LOCALE_NEUTRAL },
#define SELFREG_TYPELIB(pGuid, vMj, vMn, lcid) { SELFREG_TYPE_TYPELIB, (vMj), (vMn), (pGuid), (lcid) },
#define SELFREG_TYPELIB_END             { SELFREG_TYPE_NULL, 0, 0, nullptr, LOCALE_NEUTRAL }};}

#define SELFREG_CALLBACK_BEGIN(name)    namespace SELFREG_DATA {  const SELFREG_STRUCT_CALLBACK (name)[] = {{ SELFREG_TYPE_CALLBACK_MAP, FALSE, nullptr },
#define SELFREG_CALLBACK_END            { SELFREG_TYPE_NULL, FALSE, nullptr }};}
#define SELFREG_CALLBACK_ONREGSERVER(func)  { SELFREG_TYPE_CALLBACK, TRUE, (func) },
#define SELFREG_CALLBACK_ONUNREGSERVER(func){ SELFREG_TYPE_CALLBACK, FALSE, (func) },

#define SELFREG_VALUE_FILENAME          MAKEINTRESOURCE(1)

#endif
