#include "precomp.h"

HRESULT AskForFileName(HWND hWndParent, int idEdit, UINT idFilter, REFGUID client)
{
    COMDLG_FILTERSPEC* pSpec = nullptr;
    IFileDialog* pfd = nullptr;
    OPENFILENAME ofn;
    HRESULT hr;
    TCHAR szFilter[128], szFileName[MAX_PATH * 2];
    LPTSTR pszTemp;

    GetDlgItemText(hWndParent, idEdit, szFileName, _countof(szFileName));

    if(FAILED(CoCreateInstance(CLSID_FileOpenDialog, nullptr, CLSCTX_SERVER, IID_IFileOpenDialog, reinterpret_cast<void**>(&pfd))))
    {
        ZeroMemory(&ofn, sizeof(ofn));
        ofn.lStructSize     = sizeof(ofn);
        ofn.hwndOwner       = hWndParent;
        ofn.hInstance       = g_hInst;
        ofn.lpstrFilter     = szFilter;
        ofn.nFilterIndex    = 1;
        ofn.lpstrFile       = szFileName;
        ofn.nMaxFile        = _countof(szFileName);
        ofn.Flags           = OFN_ENABLESIZING | OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_PATHMUSTEXIST;

        LoadString(g_hInst, idFilter, szFilter, _countof(szFilter));

        for(pszTemp = szFilter; *pszTemp; ++pszTemp)
            if(*pszTemp == TEXT('|'))
                *pszTemp = TEXT('\0');

        if(!GetOpenFileName(&ofn))
            return((CommDlgExtendedError()) ? E_FAIL : S_FALSE);

        SetDlgItemText(hWndParent, idEdit, szFileName);
        hr = S_OK;
    }
    else
    {
        unsigned x, cnt = 0;
        pfd->SetClientGuid(client);

        if(LoadString(g_hInst, idFilter, szFilter, _countof(szFilter)))
        {
            for(x = 0; szFilter[x]; ++x)
                if(szFilter[x] == TEXT('|'))
                    ++cnt;

            if(cnt % 2)
                cnt = 0;
            else
                cnt /= 2;
        }

        if(nullptr != (pSpec = ((cnt) ? (new (std::nothrow) COMDLG_FILTERSPEC[cnt]) : nullptr)))
        {
            unsigned idx = 0;
            ZeroMemory(pSpec, sizeof(*pSpec) * (cnt));
            pSpec->pszName = pszTemp = szFilter;
            x = 0;

            while(*pszTemp)
            {
                if(*pszTemp == TEXT('|'))
                {
                    *pszTemp = TEXT('\0');

                    if(idx < cnt)
                    {
                        if(++x % 2)
                        {
                            pSpec[x / 2].pszSpec = pszTemp + 1;
                            ++idx;
                        }
                        else
                        {
                            pSpec[x / 2].pszName = pszTemp + 1;
                        }
                    }
                }
                ++pszTemp;
            }

            ofn.nFilterIndex = 0;
            ofn.lpstrFileTitle = PathFindFileName(szFileName);

            if(*ofn.lpstrFileTitle)
            {
                for(idx = 0; idx < cnt; ++idx)
                {
                    if(PathMatchSpec(ofn.lpstrFileTitle, pSpec[idx].pszSpec))
                    {
                        ofn.nFilterIndex = idx;
                        break;
                    }
                }
            }

            if(SUCCEEDED(hr = pfd->SetFileTypes(cnt, pSpec)))
            {
                if(SUCCEEDED(hr = pfd->SetFileTypeIndex(ofn.nFilterIndex + 1)))
                {
                    if(SUCCEEDED(hr = pfd->SetFileName(ofn.lpstrFileTitle)))
                    {
                        if(SUCCEEDED(hr = pfd->SetDefaultExtension(L"")))
                        {
                            if(SUCCEEDED(hr = pfd->Show(hWndParent)))
                            {
                                IShellItem* pItem = nullptr;

                                if(SUCCEEDED(hr = pfd->GetResult(&pItem)))
                                {
                                    if(SUCCEEDED(hr = pItem->GetDisplayName(SIGDN_FILESYSPATH, &pszTemp)))
                                    {
                                        SetDlgItemTextW(hWndParent, idEdit, pszTemp);
                                        CoTaskMemFree(pszTemp);
                                    }
                                    pItem->Release();
                                }
                            }
                            else
                            {
                                if(hr == __HRESULT_FROM_WIN32(ERROR_CANCELLED))
                                    hr = S_FALSE;
                            }
                        }
                    }
                }
            }
            delete [] pSpec;
        }
        else
        {
            hr = E_FAIL;
        }
        pfd->Release();
    }

    return(hr);
}
