#include "precomp.h"

namespace DpiAwarenessHelper
{
    GETSYSTEMMETRICSFORDPI GetSystemMetricsForDpi;
    GETDPIFORWINDOW GetDpiForWindow;
    GETDPIFORSYSTEM GetDpiForSystem;

    static UINT WINAPI GetDpiForWindowStub(_In_opt_ HWND hWnd)
    {
        UINT dpi = 0;
        HDC hdc = GetDC(hWnd);

        if(hdc)
        {
            const UINT tmp = dpi = static_cast<UINT>(GetDeviceCaps(hdc, LOGPIXELSX));
            dpi += static_cast<UINT>(GetDeviceCaps(hdc, LOGPIXELSY));

            if(tmp && (dpi > tmp))
                dpi /= 2;

            ReleaseDC(hWnd, hdc);
        }

        return((dpi) ? dpi : USER_DEFAULT_SCREEN_DPI);
    }

    static UINT WINAPI GetDpiForSystemStub()
    {
        return(GetDpiForWindowStub(nullptr));
    }

    static int WINAPI GetSystemMetricsForDpiStub(_In_ int nIndex, _In_ UINT dpi)
    {
        return((GetSystemMetrics(nIndex) * static_cast<int>(dpi)) / static_cast<int>(GetDpiForSystem()));
    }

    bool InitializeDpiAwarenessHelper()
    {
        if(GetSystemMetricsForDpi)
            return(true);

        HINSTANCE hDll = GetModuleHandle(TEXT("user32.dll"));

#pragma warning(push)
#pragma warning(disable: 4191)

        if(hDll)
        {
            GetSystemMetricsForDpi = reinterpret_cast<GETSYSTEMMETRICSFORDPI>(GetProcAddress(hDll, "GetSystemMetricsForDpi"));
            GetDpiForWindow = reinterpret_cast<GETDPIFORWINDOW>(GetProcAddress(hDll, "GetDpiForWindow"));
            GetDpiForSystem = reinterpret_cast<GETDPIFORSYSTEM>(GetProcAddress(hDll, "GetDpiForSystem"));
        }

#pragma warning(pop)

        if(!GetSystemMetricsForDpi)
            GetSystemMetricsForDpi = GetSystemMetricsForDpiStub;

        if(!GetDpiForWindow)
            GetDpiForWindow = GetDpiForWindowStub;

        if(!GetDpiForSystem)
            GetDpiForSystem = GetDpiForSystemStub;

        return(true);
    }
}
