#include "precomp.h"

static LRESULT CALLBACK EmbedControl_SubclassProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, UINT_PTR uIdSubclass, DWORD_PTR dwRefData)
{
    POINT   pt;
    RECT    rc;
    LRESULT lResult = DefSubclassProc(hWnd, uMsg, wParam, lParam);

    switch(uMsg)
    {
        case WM_NCCALCSIZE:
            if(wParam)
            {
                NCCALCSIZE_PARAMS* pnccs = reinterpret_cast<NCCALCSIZE_PARAMS*>(lParam);
                GetWindowRect(reinterpret_cast<HWND>(dwRefData), &rc);
                rc.right -= rc.left;
                pnccs->rgrc[0].right -= rc.right;
                MoveWindow(reinterpret_cast<HWND>(dwRefData), pnccs->rgrc[0].right, pnccs->rgrc[0].top, rc.right, pnccs->rgrc[0].bottom - pnccs->rgrc[0].top, TRUE);
            }
            break;
        case WM_NCHITTEST:
            if(lResult == HTNOWHERE)
            {
                GetWindowRect(reinterpret_cast<HWND>(dwRefData), &rc);
                POINTSTOPOINT(pt, lParam);

                if(PtInRect(&rc, pt))
                    lResult = HTTRANSPARENT;
            }
            break;
        case WM_MOVE:
            SetWindowPos(hWnd, nullptr, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
            break;
        case WM_DESTROY:
            RemoveWindowSubclass(hWnd, EmbedControl_SubclassProc, uIdSubclass);
            break;
        default:
            break;
    }

    return(lResult);
}

BOOL EmbedControl(HWND hWnd, HWND hWndToEmbed)
{
    if(!SetWindowSubclass(hWnd, EmbedControl_SubclassProc, reinterpret_cast<UINT_PTR>(hWndToEmbed), reinterpret_cast<DWORD_PTR>(hWndToEmbed)))
        return(FALSE);

    // Enforce WM_NCCALCSIZE
    return(SetWindowPos(hWnd, nullptr, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED));
}

HRESULT MyPathCchAppend(PTSTR pszPath, size_t cchPath, PCTSTR pszMore)
{
    if((_tcslen(pszPath) + _tcslen(pszMore) + 2) > cchPath)
        return(E_FAIL);

    return((PathAppend(pszPath, pszMore)) ? S_OK : E_FAIL);
}

BOOL GetSrf04EbFirmwareDir(PTSTR pPath, DWORD cchBuf)
{
    DWORD tmp = cchBuf;

    // TI's setup creation has been changed to utilize NSIS beginning with Version 1.12.8,
    // hence MsiLocateComponent will succeed only with an earlier version installed.
    if(INSTALLSTATE_LOCAL == MsiLocateComponent(TEXT("{E188F81A-2DE3-4D52-ABC1-767EB9E03816}"), pPath, &tmp))
        return(TRUE);

    HKEY hKey;
    DWORD dwType = REG_NONE;
    BOOL ret = FALSE;

    if(cchBuf)
    {
        if(ERROR_SUCCESS == RegOpenKeyEx(HKEY_LOCAL_MACHINE, REGSTR_PATH_UNINSTALL TEXT("\\Texas Instruments Flash Programmer"), 0, STANDARD_RIGHTS_READ | KEY_QUERY_VALUE | KEY_WOW64_32KEY, &hKey))
        {
            tmp = (cchBuf - 1) * sizeof(TCHAR);

            if(ERROR_SUCCESS == RegQueryValueEx(hKey, INSTALLPROPERTY_INSTALLLOCATION, nullptr, &dwType, reinterpret_cast<BYTE*>(pPath), &tmp))
            {
                if(dwType == REG_SZ)
                {
                    pPath[tmp / 2] = TEXT('\0');
                    PathUnquoteSpaces(pPath);

                    if(SUCCEEDED(MyPathCchAppend(pPath, cchBuf, TEXT("Firmware\\SmartRF04EB"))))
                        ret = PathFileExists(pPath);
                }
            }

            RegCloseKey(hKey);
        }
    }

    return(ret);
}

static int MinGetSystemMetricsForDpi(__in int nIndex1, __in int nIndex2, UINT dpi)
{
    const int val1 = DpiAwarenessHelper::GetSystemMetricsForDpi(nIndex1, dpi);
    const int val2 = DpiAwarenessHelper::GetSystemMetricsForDpi(nIndex2, dpi);
    return(min(val1, val2));
}

static HICON LoadClassIcon(REFGUID rcls)
{
    SP_CLASSIMAGELIST_DATA  spimld = {};
    HICON                   hIcon = nullptr;
    INT                     nIndex;

    spimld.cbSize = sizeof(spimld);

    if(!SetupDiGetClassImageList(&spimld))
        return(nullptr);

    if(SetupDiGetClassImageIndex(&spimld, &rcls, &nIndex))
        hIcon = ImageList_GetIcon(spimld.ImageList, nIndex, ILD_TRANSPARENT);

    SetupDiDestroyClassImageList(&spimld);
    return(hIcon);
}

void InitializeDebugAdaptersCB(HWND hWndCB)
{
    COMBOBOXEXITEMA cbexi = {};
    const UINT      dpi = DpiAwarenessHelper::GetDpiForWindow(hWndCB);
    HIMAGELIST      himl;
    LRESULT         selected, idx;
    DWORD           dwDevices = 0;
    CHAR            szPort[256];
    int             iCommport = -1, iUsb = -1;

    szPort[0] = '\0';

    if(0 <= (selected = SendMessage(hWndCB, CB_GETCURSEL, 0, 0)))
    {
        selected = SendMessage(hWndCB, CB_GETITEMDATA, static_cast<WPARAM>(selected), 0);

        if(!IS_INTRESOURCE(selected) && (selected > 0))
            StringCchCopyA(szPort, _countof(szPort), reinterpret_cast<const char*>(selected));
    }

    SendMessage(hWndCB, CB_RESETCONTENT, 0, 0);

    if(nullptr != (himl = ImageList_Create(MinGetSystemMetricsForDpi(SM_CXSMICON, SM_CXMENUCHECK, dpi), MinGetSystemMetricsForDpi(SM_CYSMICON, SM_CYMENUCHECK, dpi), ILC_MASK | ILC_COLOR32, 2, 0)))
    {
        HICON hIcon;

        if(nullptr != (hIcon = LoadClassIcon(GUID_DEVCLASS_PORTS)))
        {
            iCommport = ImageList_AddIcon(himl, hIcon);
            DestroyIcon(hIcon);
        }

        if(nullptr != (hIcon = LoadClassIcon(GUID_DEVCLASS_USB)))
        {
            iUsb = ImageList_AddIcon(himl, hIcon);
            DestroyIcon(hIcon);
        }

        if(nullptr != (himl = reinterpret_cast<HIMAGELIST>(SendMessage(hWndCB, CBEM_SETIMAGELIST, 0, reinterpret_cast<LPARAM>(himl)))))
            ImageList_Destroy(himl);
    }

    cbexi.mask = CBEIF_IMAGE | CBEIF_LPARAM | CBEIF_SELECTEDIMAGE | CBEIF_TEXT;
    cbexi.iItem = -1;
    cbexi.iImage = iUsb;
    cbexi.iSelectedImage = iUsb;
    USBDebugDevices(&dwDevices);

    for(DWORD dwCurrent = 0; dwCurrent < dwDevices; ++dwCurrent)
    {
        if(FAILED(GetUSBDeviceSN(dwCurrent, const_cast<const char**>(&cbexi.pszText))))
            continue;

        cbexi.lParam = reinterpret_cast<LPARAM>(cbexi.pszText);

        if(0 <= (idx = SendMessage(hWndCB, CBEM_INSERTITEMA, 0, reinterpret_cast<LPARAM>(&cbexi))))
            if(!lstrcmpiA(cbexi.pszText, szPort))
                SendMessage(hWndCB, CB_SETCURSEL, static_cast<WPARAM>(idx), 0);
    }

    cbexi.iImage = iCommport;
    cbexi.iSelectedImage = iCommport;
    cbexi.pszText = szPort;

    for(int port = 1; port <= 1024; ++port)
    {
        if(FAILED(StringCchPrintfA(szPort, _countof(szPort), "COM%d", port)))
            continue;

        QueryDosDeviceA(szPort, nullptr, 0);

        if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
            continue;

        cbexi.lParam = port;

        if(0 <= (idx = SendMessage(hWndCB, CBEM_INSERTITEMA, 0, reinterpret_cast<LPARAM>(&cbexi))))
            if(selected == port)
                SendMessage(hWndCB, CB_SETCURSEL, static_cast<WPARAM>(idx), 0);
    }

    if(0 > SendMessage(hWndCB, CB_GETCURSEL, 0, 0))
        SendMessage(hWndCB, CB_SETCURSEL, 0, 0);
}

void SelectDebugAdapter(HWND hWndCB, CSiUtil& util)
{
    WPARAM idx;
    LRESULT data;

    util.Disconnect();
    InitializeDebugAdaptersCB(hWndCB);

    for(idx = 0; (CB_ERR != (data = SendMessage(hWndCB, CB_GETITEMDATA, idx, 0))); ++idx)
    {
        if(util == reinterpret_cast<const char*>(data))
        {
            SendMessage(hWndCB, CB_SETCURSEL, idx, 0);
            break;
        }
    }
}
